--@desc: define libs
	Acolyte 	= AceLibrary("AceAddon-2.0"):new("AceConsole-2.0","AceEvent-2.0")
	AcoDewdrop 	= AceLibrary("Dewdrop-2.0")

--@desc: localization
if (GetLocale() == "deDE") then
	AcoLoc = AcoLoc_deDE
else
	AcoLoc = AcoLoc_enUS
end

--@desc: determinate where to output messages
if IsAddOnLoaded("DreamChat") then CHATFRAME = DreamChat1 else CHATFRAME = DEFAULT_CHAT_FRAME end

--@desc: recent revision
Acolyte_Revision = 200701291739 -- YYYY MM DD hh mm

--@desc: default values
Acolyte_Defaults = {
["AcolyteDB"] = {
	--@note: StoneCreator
	["showSpellStone"] 				= false,
	["showFireStone"] 				= false,
	["showHealthStone"] 			= true,
	["showSoulStone"] 				= true,
	--@note: MinionSummoner
	["lastMinion"] 					= "Imp",
	--@note: NightFall
	["flashFrame"] 					= true,
	["showIcon"] 					= true,
	["chatWarning"] 				= true,
	--@note: SoulKeeper
	["ShardCountBag"] 				= 100,
	["OutOfShardMsg"] 				= true,
	["NumberOfShardsToWarn"] 		= 20,
	--@note: General
	["locked"] 						= false,
	["firstload"]					= true,
    ["Scale"] 						= 1,
    ["Debug"] 						= false,
	["PopupDirection"] 				= "BOTTOMLEFT",
	["x"] 							= 300,
	["y"] 							= 300,
	["MinionSummonerEnabled"] 		= true,
	["SpellCasterEnabled"] 			= true,
	["StoneCreatorEnabled"]			= true,
	["MountSummonerEnabled"] 		= true,
	["NightFallEnabled"] 			= true,
	["RitualOfSummoningEnabled"] 	= true,
	["SoulKeeperEnabled"] 			= true,
	["ButtonsPerZeile"]				= 5,
	SpellDB = {},
	["Revision"] 					= Acolyte_Revision,
  },
}

--@desc: Initialize (AceAddOn)
function Acolyte:OnInitialize()
	--@desc: DB will be created if none exists
  if (not AcolyteDB) then
    AcolyteDB = Acolyte_Defaults["AcolyteDB"]
  end
  
	--@desc: DB will be updated if recent revision is newer then the one before installed
	if (AcolyteDB["Revision"] < Acolyte_Revision) then
	  Acolyte_Temp = Acolyte_Defaults["AcolyteDB"]
	  for k,v in pairs(AcolyteDB) do
	    if (Acolyte_Defaults["AcolyteDB"][k]) then
	      Acolyte_Temp[k] = v
	    end
	  end
	  Acolyte_Temp["Revision"] = Acolyte_Revision
	  AcolyteDB = Acolyte_Temp
	end
  
end


--@desc: Ace2 function
function Acolyte:OnEnable()
	Acolyte:LoadLayout() --@note: load the positions of the buttons

	if AcolyteDB.firstload == true then Acolyte:SPELLS_CHANGED() AcolyteDB.firstload = false end
	
	self:RegisterEvent("SPELLS_CHANGED")	
	
	--@note: Ritual of Summoning related
	self:RegisterEvent("UNIT_SPELLCAST_SUCCEEDED")
	
	--@note: NightFall related
	--@note: events will only be registered if this part of the addon is enabled
		if (NightFallEnabled == true) then
			self:RegisterEvent("CHAT_MSG_SPELL_PERIODIC_SELF_BUFFS")
			self:RegisterEvent("CHAT_MSG_SPELL_AURA_GONE_SELF")
			self:RegisterEvent("PLAYER_DEAD")
		end
		
	--@note: SoulKeeper related
		if (AcolyteDB.SoulKeeperEnabled == true) then
			self:RegisterEvent("BAG_UPDATE")
		end
	
	--@note: watch event to determinate when spellcasting is impossible
	self:RegisterEvent("PLAYER_REGEN_DISABLED")

	--@desc: button for moving the other buttons will be created
		Acolyte_MoveButton = CreateFrame("Button", "Acolyte_MoveButton", UIParent, "SecureActionButtonTemplate")
	Acolyte_MoveButton:SetFrameStrata("TOOLTIP")
	Acolyte_MoveButton:SetWidth(16)
	Acolyte_MoveButton:SetHeight(16)
	Acolyte_MoveButton:SetMovable(true)
	Acolyte_MoveButton:EnableMouse(true)
	Acolyte_MoveButton:SetClampedToScreen(true) --@note: with this, the user can't move the move button off the screen
	Acolyte_MoveButton:RegisterForDrag("LeftButton")
	Acolyte_MoveButton:RegisterForClicks("RightButtonDown", "LeftButtonDown")
	Acolyte_MoveButton:SetScale(AcolyteDB.Scale)
		Acolyte_MoveButtonTexture = Acolyte_MoveButton:CreateTexture(nil,"TOOLTIP")
	Acolyte_MoveButtonTexture:SetTexture("Interface\\AddOns\\Acolyte\\Images\\MoveButton")
	Acolyte_MoveButtonTexture:SetAllPoints(Acolyte_MoveButton)
	Acolyte_MoveButton.texture = Acolyte_MoveButtonTexture
	Acolyte_MoveButton:SetPoint("LEFT",UIParent,"BOTTOMLEFT",AcolyteDB["x"]/UIParent:GetEffectiveScale()/AcolyteDB["Scale"]-10,AcolyteDB["y"]/UIParent:GetEffectiveScale()/AcolyteDB["Scale"])
	lastParent = Acolyte_MoveButton
	ButtonsCreated = 1

	--@desc: position will be saved if the above button moves
	Acolyte_MoveButton:SetScript("OnMouseDown", function() 		 
			if ( ( ( not this.isLocked ) or ( this.isLocked == 0 ) ) and ( arg1 == "LeftButton" ) ) then
			 this:ClearAllPoints()
			 this:SetMovable(1)
			 this:StartMoving()
			 this.isMoving = true
			 end 
							end)
							
	Acolyte_MoveButton:SetScript("OnMouseUp", function() 		 
			if ( this.isMoving ) then
			  this:StopMovingOrSizing()
			  this.isMoving = false
			  AcolyteDB["x"],AcolyteDB["y"] = GetCursorPosition(UIParent)
			 end
							end)
												
	Acolyte_MoveButton:SetScript("PostClick", function() 							
			Acolyte_LoadOptions() 
			AcoDewdrop:Open( UIParent, 
				'children', Acolyte_Options, 
				'cursorX', true, 'cursorY', true )
	end)
				
		
	--@desc: default is unlocked
	if (AcolyteDB["locked"] == false) then
		Acolyte_MoveButton:Show()
	else
		Acolyte_MoveButton:Hide()
	end

	--@desc: Der Button fr die beschwrung der Diener wird erstellt
	Acolyte:CreateMinnionSummoner()

	--@desc: Chat Kommandos werden erstellt (AceConsole-2.0)
		Acolyte:RegisterChatCommand({ "/aco", "/acolyte", }, {
		type = "execute", name = "Acolyte", desc = "Acolyte",
		func = function() 
			Acolyte_LoadOptions() 
			AcoDewdrop:Open( UIParent, 
				'children', Acolyte_Options, 
				'cursorX', true, 'cursorY', true )  
		end,
	})
	
		--@desc: hooking the WorldFrame to hide all Popups on Mouseklick (from WoWWiki)
		local oldFunc = WorldFrame:GetScript("OnMouseDown")
		if ( oldFunc ) then --@note: hook the old function if one already exists
		   WorldFrame:SetScript("OnMouseDown", function() oldFunc() Acolyte:HideAllPopups() end )
		else
		   WorldFrame:SetScript("OnMouseDown", function() Acolyte:HideAllPopups() end)
		end
		
end

--@desc: function to hide all visible Popups
function Acolyte:HideAllPopups()
	if (not UnitAffectingCombat("player")) then
		Acolyte:HideStonesPopup()
		Acolyte:HideShieldSensePopup()
		Acolyte:HideMinionSummonerPopup()
	end
end 

--@desc: hides all popups before a combat starts
function Acolyte:PLAYER_REGEN_DISABLED()
		Acolyte:HideStonesPopup()
		Acolyte:HideShieldSensePopup()
		Acolyte:HideMinionSummonerPopup()
end 

--@desc: determinate if the player got a specific spell
function Acolyte:PlayerHasSpell(Spell)
	local i = 1
	while true do
	   local spellName = GetSpellName(i, BOOKTYPE_SPELL)
	   if not spellName then
	      do break end
	   end
	   
		if string.find(spellName, Spell) then 
				return true
		end
	   i = i + 1
	end
end 

--[[
"LEARNED_SPELL_IN_TAB"
	Category: Spell
  	

Fired when a new spell/ability is added to the spellbook. e.g. When training a new or a higher level spell/ability.

arg1
    Number of the tab which the spell/ability is added to 
	
	
##################################################################################################	
	##################################################################################################
	
	
"SPELLS_CHANGED"
	Category: Spell
  	

Fired when:

    * Spellbook initially loads on client initialization. (Attempting to use GetSpellName before this is complete will result in a UI error. Note: This event happens after ADDON_LOADED and VARIABLES_LOADED.)
    * Learning new stuff that goes in the spellbook.
    * Opening the spellbook.
    * Equipping/Removing/Swapping primary weapon. (Changes the Attack icon) 

arg1
    nil when the char learns stuff. Also when changing weapons and shapeshifting. In addition it seems to be called on regular intervals for no apparent reason.

       1. when the user opens the spellbook. 

    'LeftButton' when using the mouse to open the spellbook or to browse through the pages and tabs of the open spellbook. 	
--]]
function Acolyte:SPELLS_CHANGED()

	if Acolyte:PlayerHasSpell(AcoLoc.SummonImp) then AcolyteDB.SpellDB.SummonImpAvailable = true else AcolyteDB.SpellDB.SummonImpAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SummonVoidwalker) then AcolyteDB.SpellDB.SummonVoidwalkerAvailable = true else AcolyteDB.SpellDB.SummonVoidwalkerAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SummonSuccubus) then AcolyteDB.SpellDB.SummonSuccubusAvailable = true else AcolyteDB.SpellDB.SummonSuccubusAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SummonFelhunter) then AcolyteDB.SpellDB.SummonFelhunterAvailable = true else AcolyteDB.SpellDB.SummonFelhunterAvailable = false end

	if Acolyte:PlayerHasSpell(AcoLoc.SummonFelguard) then AcolyteDB.SpellDB.SummonFelguardAvailable = true else AcolyteDB.SpellDB.SummonFelguardAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.EyeOfKilrogg) then AcolyteDB.SpellDB.EyeOfKilroggAvailable = true else AcolyteDB.SpellDB.EyeOfKilroggAvailable = false end

	if Acolyte:PlayerHasSpell(AcoLoc.FelArmor) then AcolyteDB.SpellDB.FelArmorAvailable = true else AcolyteDB.SpellDB.FelArmorAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.DemonArmor) then AcolyteDB.SpellDB.DemonArmorAvailable = true else AcolyteDB.SpellDB.DemonArmorAvailable = false end

	if Acolyte:PlayerHasSpell(AcoLoc.DemonSkin) then AcolyteDB.SpellDB.DemonSkinAvailable = true else AcolyteDB.SpellDB.DemonSkinAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SenseDemons) then AcolyteDB.SpellDB.SenseDemonsAvailable = true else AcolyteDB.SpellDB.SenseDemonsAvailable = false end

	if Acolyte:PlayerHasSpell(AcoLoc.UnendingBreath) then AcolyteDB.SpellDB.UnendingBreathAvailable = true else AcolyteDB.SpellDB.UnendingBreathAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.DetectInvisibility) then AcolyteDB.SpellDB.DetectInvisibilityAvailable = true else AcolyteDB.SpellDB.DetectInvisibilityAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.RitualOfSummoning) then AcolyteDB.SpellDB.RitualOfSummoningAvailable = true else AcolyteDB.SpellDB.RitualOfSummoningAvailable = false end

	if Acolyte:PlayerHasSpell(AcoLoc.SummonDreadsteed) then AcolyteDB.SpellDB.SummonDreadsteedAvailable = true else AcolyteDB.SpellDB.SummonDreadsteedAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SummonFelsteed) then AcolyteDB.SpellDB.SummonFelsteedAvailable = true else AcolyteDB.SpellDB.SummonFelsteedAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.DrainSoul) then AcolyteDB.SpellDB.DrainSoulAvailable = true else AcolyteDB.SpellDB.DrainSoulAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.HealthStone) then AcolyteDB.SpellDB.HealthStoneAvailable = true else AcolyteDB.SpellDB.HealthStoneAvailable = false end
	
	if Acolyte:PlayerHasSpell(AcoLoc.SoulStone) then AcolyteDB.SpellDB.SoulStoneAvailable = true else AcolyteDB.SpellDB.SoulStoneAvailable = false end	
	
	if Acolyte:PlayerHasSpell(AcoLoc.SpellStone) then AcolyteDB.SpellDB.SpellStoneAvailable = true else AcolyteDB.SpellDB.SpellStoneAvailable = false end	

	if Acolyte:PlayerHasSpell(AcoLoc.FireStone) then AcolyteDB.SpellDB.FireStoneAvailable = true else AcolyteDB.SpellDB.FireStoneAvailable = false end	
	
end 








